require "em-synchrony/em-http"
require "kameleoon/version"

module Kameleoon
  # @api private
  module Request
    protected
    API_URL = "https://api.kameleoon.com"

    module Method
      GET = "get"
      POST = "post"
    end

    def get(request_options, url = API_URL, connexion_options = {})
      request(Method::GET, request_options, url, connexion_options)
    end

    def post(request_options, url = API_URL, connexion_options = {})
      request(Method::POST, request_options, url, connexion_options)
    end

    private

    def request(method, request_options, url, connexion_options)
      connexion_options[:tls] = {verify_peer: false}
      add_user_agent(request_options)
      case method
        when Method::POST then
          return EventMachine::HttpRequest.new(url, connexion_options).apost request_options
        when Method::GET then
          return EventMachine::HttpRequest.new(url, connexion_options).aget request_options
        else
          print "Unknown request type"
          return false
      end
    end

    def is_successful(request)
      !request.nil? && request != false && /20\d/.match(request.response_header.status.to_s)
    end

    def add_user_agent(request_options)
      if request_options[:head].nil?
        request_options[:head] = {'Kameleoon-Client' => 'sdk/ruby/' + Kameleoon::VERSION}
      else
        request_options[:head].store('Kameleoon-Client', 'sdk/ruby/' + Kameleoon::VERSION)
      end
    end
  end
end


