require 'kameleoon/targeting/tree_builder'
require 'kameleoon/exceptions'

module Kameleoon
  #@api private
  module Targeting
    class Segment
      include TreeBuilder
      attr_accessor :id, :tree
      def to_s
        print("\nSegment id: " + @id.to_s)
        print("\n")
        @tree.to_s
      end

      def initialize(*args)
        unless args.empty?
          if args.length == 1
            hash = args.first
            if hash.nil?
              raise Kameleoon::Exception::NotFound.new("arguments for segment")
            end
            if hash["id"].nil?
              raise Kameleoon::Exception::NotFound.new("id")
            end
            @id = hash["id"]
            if hash["conditionsData"].nil?
              raise Kameleoon::Exception::NotFound.new(hash["conditionsData"])
            end
            @tree = create_tree(hash["conditionsData"])
          elsif args.length == 2
            @id = args[0]
            @tree = args[1]
          end
        end
      end

      def check_tree(targeting_data)
        if @tree.nil?
          is_targeted = true
        else
          is_targeted = @tree.check(targeting_data)
        end
        is_targeted == true
      end
    end

    class Tree
      attr_accessor :or_operator, :left_child, :right_child, :condition

      def to_s
        print("or_operator: " + @or_operator.to_s)
        print("\n")
        print("condition: " + @condition.to_s)
        unless @left_child.nil?
          print("\n")
          print("Left child:\n    ")
          @left_child.to_s
        end
        unless @right_child.nil?
          print("\n")
          print("right child:\n    ")
          @right_child.to_s
        end
      end

      def initialize(or_operator = nil, left_child = nil, right_child = nil, condition = nil)
        @or_operator = Marshal.load(Marshal.dump(or_operator))
        @left_child = left_child
        @right_child = right_child
        @condition = condition
      end

      def check(datas)
        unless @condition.nil?
          is_targeted = check_condition(datas)
        else
          if @left_child.nil?
            is_left_child_targeted = true
          else
            is_left_child_targeted = @left_child.check(datas)
          end

          if is_left_child_targeted.nil?
            has_to_compute_right_child = true
          else
            has_to_compute_right_child = (is_left_child_targeted != @or_operator)
          end

          # Compute right child tree
          is_right_child_targeted = nil
          if has_to_compute_right_child
            if @right_child.nil?
              is_right_child_targeted = true
            else
              is_right_child_targeted = @right_child.check(datas)
            end
          end

          # Computing results
          if is_left_child_targeted.nil?
            if is_right_child_targeted == @or_operator
              is_targeted = Marshal.load(Marshal.dump(@or_operator)) #Deep copy
            else
              is_targeted = nil
            end
          else
            if is_left_child_targeted == @or_operator
              is_targeted = Marshal.load(Marshal.dump(@or_operator)) #Deep copy
            else
              if is_right_child_targeted == true
                is_targeted = true
              elsif is_right_child_targeted == false
                is_targeted = false
              else
                is_targeted = nil
              end
            end
          end
        end
        Marshal.load(Marshal.dump(is_targeted)) #Deep copy
      end

      def check_condition(datas, condition = @condition)
        if condition.nil?
          is_targeted = true
        else
          is_targeted = condition.check(datas)
          unless condition.include
            if is_targeted.nil?
              is_targeted = true
            else
              is_targeted = !is_targeted
            end
          end
        end
        Marshal.load(Marshal.dump(is_targeted)) #Deep copy
      end
    end

    module DataType
      CUSTOM = "CUSTOM"
    end

    module ConditionType
      CUSTOM_DATUM = "CUSTOM_DATUM"
    end

    module Operator
      UNDEFINED = "UNDEFINED"
      CONTAINS = "CONTAINS"
      EXACT = "EXACT"
      MATCH = "REGULAR_EXPRESSION"
      LOWER = "LOWER"
      EQUAL = "EQUAL"
      GREATER = "GREATER"
      IS_TRUE = "TRUE"
      IS_FALSE = "FALSE"
    end
  end
end